from dataclasses import dataclass
from typing import Callable
from requests_oauthlib import OAuth2Session
from etsy_apiv3.utils.RequestException import EtsyRequestException

def token_saver(token):
    return token

@dataclass(frozen=True)
class EtsyAuth:
    """
    A class to represent a etsy auth
    
    Attributes:
        headers (dict): Request Headers
        session (requests_oauthlib.OAuth2Session): Requests Oauthlib Object
    """
    
    headers: dict
    session: OAuth2Session
    __base_endpoint:str = "https://openapi.etsy.com/v3/application"
    
    def request(self, endpoint: str, method="GET", *args, **kwargs) -> dict:
        """ 
        Send Request to target endpoint by method
        
        Args:
            endpoint (str): Api Endpoint Url
            method (str, optional): HTTP Methods [GET, POST, PUT, DELETE, UPDATE]. Defaults to "GET".

        Raises:
            EtsyRequestException: EtsyRequestException(status_code, message)

        Returns:
            json: Json From Request Response
        """

        url = f"{self.__base_endpoint}/{endpoint}"
        req = self.session.request(method, url, headers=self.headers, *args, **kwargs)

        response: dict = req.json()
        if "error" in response.keys():
            raise EtsyRequestException(req.status_code, response["error"])
        return response



class EtsySession:
    
    def __init__(self, client_key: str, client_secret: str, token: dict, refresh_url="https://api.etsy.com/v3/public/oauth/token", token_updater: Callable = token_saver):
        self.CLIENT_KEY = client_key
        self.CLIENT_SECRET = client_secret
        self.TOKEN = token
        self.REFRESH_URL = refresh_url
        self.TOKEN_UPDATER = token_updater
        
    def create_auth(self):
        refresh_kwargs = {
            'client_id': self.CLIENT_KEY,
            'client_secret': self.CLIENT_SECRET,
        }
        
        headers = {"x-api-key": self.CLIENT_KEY}
        session = OAuth2Session(client_id=self.CLIENT_KEY, token=self.TOKEN, auto_refresh_kwargs=refresh_kwargs, auto_refresh_url=self.REFRESH_URL, token_updater=self.TOKEN_UPDATER)
        
        return EtsyAuth(headers=headers, session=session)
        



from __future__ import annotations
from pydantic import BaseModel

class User(BaseModel):
    user_id: int
    login_name: str
    primary_email: str
    first_name: str
    last_name: str
    create_timestamp: int
    referred_by_user_id: int
    use_new_inventory_endpoints: bool
    is_seller: bool
    bio: str
    gender: str
    birth_month: str
    birth_day: str
    transaction_buy_count: int
    transaction_sold_count: int


from dataclasses import dataclass
from etsy_apiv3.utils.APIV3 import EtsyAuth
from etsy_apiv3.utils.Response import Response
from etsy_apiv3.models.Review import Review

@dataclass
class ReviewResource:
    auth: EtsyAuth
    
    def get_reviews_by_listing_id(self, listing_id: int, limit: int = 25, offset: int = 0):
        
        params = {"limit": limit, "offset": offset}
        url = f"listings/{listing_id}/reviews"
        
        response = self.auth.request(url, params=params)
        return Response[Review](**response)
    
    def get_reviews_by_shop_id(self, shop_id: int, limit: int = 25, offset: int = 0):
        
        params = {"limit": limit, "offset": offset}
        url = f"shops/{shop_id}/reviews"
        
        response = self.auth.request(url, params=params)
        return Response[Review](**response)


from __future__ import annotations
from etsy_apiv3.models.ListingImage import ListingImage
from etsy_apiv3.models.ProductionPartner import ProductionPartner
from etsy_apiv3.models.ShippingProfile import ShippingProfile
from etsy_apiv3.models.Shop import Shop
from etsy_apiv3.models.Translation import Translation
from etsy_apiv3.models.User import User
from etsy_apiv3.models.Price import Price
from typing import List, Optional
from pydantic import BaseModel

class Listing(BaseModel):
    listing_id: int
    user_id: int
    shop_id: int
    title: str
    description: str
    state: str
    creation_timestamp: int
    ending_timestamp: int
    original_creation_timestamp: int
    last_modified_timestamp: int
    state_timestamp: int
    quantity: int
    shop_section_id: Optional[int]
    featured_rank: int
    url: str
    num_favorers: int
    non_taxable: bool
    is_customizable: bool
    is_personalizable: bool
    personalization_is_required: bool
    personalization_char_count_max: Optional[int]
    personalization_instructions: Optional[str]
    listing_type: str
    tags: List[str]
    materials: List[str]
    shipping_profile_id: Optional[int]
    processing_min: Optional[int]
    processing_max: Optional[int]
    who_made: Optional[str]
    when_made: Optional[str]
    is_supply: bool
    item_weight: Optional[int]
    item_weight_unit: Optional[str]
    item_length: Optional[int]
    item_width: Optional[int]
    item_height: Optional[int]
    item_dimensions_unit: Optional[str]
    is_private: bool
    style: List[str]
    file_data: str
    has_variations: bool
    should_auto_renew: bool
    language: str
    price: Price
    taxonomy_id: int
    shipping_profile: Optional[ShippingProfile]
    user: Optional[User]
    shop: Optional[Shop]
    images: Optional[List[ListingImage]]
    production_partners: List[ProductionPartner]
    skus: List[str]
    translations: Optional[List[Translation]]
    
    



from dataclasses import dataclass
from etsy_apiv3.utils.APIV3 import EtsyAuth
from etsy_apiv3.utils.Response import Response
from etsy_apiv3.models.Receipt import Receipt, ReceiptType

@dataclass
class ReceiptResource:
    """
    Receipt Resource is the utility class to get receipts from Etsy
    
    Attributes:
        auth (EtsyAuth): EtsyAuth Object
        
    """
    
    __endpoint = "shops/{shop_id}/receipts"
    auth: EtsyAuth
    
    
    
    def find(self, shop_id: int, limit=25, offset=0, type: ReceiptType = ReceiptType.UNSHIPPED) -> Response[Receipt]:
        """
        Find Receipts By Shop ID And Receipt Type

        Args:
            
            shop_id (int): Your SHOP ID
            limit (int, optional): Limit Of Receipt Objects. Defaults to 25.
            offset (int, optional): How many receipts to skip. Defaults to 0.
            type (ReceiptType, optional): Receipt type to find. Defaults to ReceiptType.UNSHIPPED.

        Returns:
            Response[Receipt]: Create Response Object Derived from the Receipt Object from json
        """
        
        endpoint = self.__endpoint.format(shop_id=shop_id)
        
        params = {"limit":limit, "offset":offset}
        params.update(type)
        json = self.auth.request(endpoint=endpoint, params=params)
        
        return Response[Receipt](**json)
    
    def find_one(self, shop_id: int, receipt_id: int):
        """
        Find One Receipt By Shop ID And Receipt ID

        Args:
            shop_id (int): Your SHOP ID
            receipt_id (int): Receipt ID
            
        Returns:
            Receipt: Create Receipt Object from json
        """
        endpoint = f"{self.__endpoint.format(shop_id=shop_id)}/{receipt_id}"
        json = self.auth.request(endpoint=endpoint)
        
        return Receipt(**json)

    def create_shipment(self, shop_id: int, receipt_id: int, tracking_number: str, carrier_name: str, send_bcc: bool = True, note_to_buyer: str = ""):
        """
        Adds tracking information to the receipt object

        Args:
            shop_id (int): SHOP ID
            receipt_id (int): Target Receipt ID
            tracking_number (str): Tracking Number
            carrier_name (str): Carrier Name Ex: UPS
            send_bcc (bool, optional): Send Mail. Defaults to True.
            note_to_buyer (str, optional): Note To Buyer From Seller. Defaults to "".

        Returns:
            Receipt: Receipt Object from json
        """
        endpoint = f"{self.__endpoint.format(shop_id=shop_id)}/{receipt_id}/tracking"
        
        data = {
            "tracking_code":tracking_number, "carrier_name":carrier_name,
            "send_bcc":send_bcc, "note_to_buyer":note_to_buyer
        }
        
        json = self.auth.request(endpoint=endpoint, method="POST", data=data)
        return Receipt(**json)


from pydantic.generics import GenericModel
from typing import Generic, TypeVar, List

ResultsT = TypeVar('ResultsT')

class Response(GenericModel, Generic[ResultsT]):
    count: int
    results: List[ResultsT]



from __future__ import annotations
from pydantic import BaseModel
from etsy_apiv3.models.Price import Price

class ShippingProfileDestination(BaseModel):
    shipping_profile_destination_id: int
    shipping_profile_id: int
    origin_country_iso: str
    destination_country_iso: str
    destination_region: str
    primary_cost: Price
    secondary_cost: Price
    shipping_carrier_id: int
    mail_class: str
    min_delivery_days: int
    max_delivery_days: int


from dataclasses import dataclass
from etsy_apiv3.models.Shop import Shop
from etsy_apiv3.utils.APIV3 import EtsyAuth
from etsy_apiv3.utils.Response import Response

@dataclass
class ShopResource:
    """
    Shop Resource Of Etsy Api V3.

    """
    auth: EtsyAuth
    
    def get_shop(self, shop_id: int):
        """
        Get Shop By Shop Id And Return A Shop Object

        Args:
            shop_id (int): Shop Id

        Returns:
            Shop: Shop Object
        """
        
        endpoint = f"shops/{shop_id}"
        response = self.auth.request(endpoint)
        return Shop(**response)
    
    def find_shops(self, shop_name: str):
        """
        Find All Shops And Return Response Object
        
        Returns:
            Response: Return Response object of shop list 
        """
        endpoint = "shops"
        response = self.auth.request(endpoint, params={"shop_name":shop_name})
        return Response[Shop](**response)

    def find_shop_by_owner_user_id(self, user_id: int):
        """
        Find Shop By Owner User Id And Return A Shop Object.

        Args:
            user_id (int): Shop Owner User Id

        Returns:
            Shop: Return A Shop Object
        """
        endpoint = f"users/{user_id}/shops"
        response = self.auth.request(endpoint)
        return Shop(**response)
    
    def update_shop(self, shop_id: int, title: str = "", announcement: str = "", sale_message: str = "", digital_sale_message: str = ""):
        endpoint = f"shops/{shop_id}"
        pass


from __future__ import annotations
from pydantic import BaseModel

class Shipment(BaseModel):
    receipt_shipping_id: int
    shipment_notification_timestamp: int
    carrier_name: str
    tracking_code: str


from __future__ import annotations
from typing import Optional
from pydantic import BaseModel

class Review(BaseModel):
    shop_id: int
    listing_id: int
    transaction_id: Optional[int]
    buyer_user_id: Optional[int]
    rating: int
    review: Optional[str]
    language: str
    image_url_fullxfull: Optional[str]
    create_timestamp: int
    update_timestamp: int
    


from dataclasses import dataclass
from typing import List, Optional
from pydantic import BaseModel
from etsy_apiv3.models.Price import Price
from etsy_apiv3.models.Transaction import Transaction
from etsy_apiv3.models.Shipment import Shipment
import pycountry

@dataclass(frozen=True)
class ReceiptType:
    UNSHIPPED = {"was_paid":True, "was_shipped":False}
    SHIPPED = {"was_paid":True, "was_shipped":True}
    UNPAID = {"was_paid":False}
    PAID = {"was_paid":True}


@dataclass(frozen=True)
class ReceiptStatus:
    CANCELED = "Canceled"
    PAID = "Paid"
    COMPLETED = "Completed"
    OPEN = "Open"
    PAYMENT_PROCESSING = "Payment Processing"
    

class Receipt(BaseModel):
    receipt_id: int
    receipt_type: int
    seller_user_id: int
    seller_email: str
    buyer_user_id: int
    buyer_email: Optional[str]
    name: str
    first_line: str
    second_line: Optional[str]
    city: Optional[str]
    state: Optional[str]
    zip: Optional[str]
    status: str
    formatted_address: str
    country_iso: str
    payment_method: str
    payment_email: str
    message_from_seller: Optional[str]
    message_from_buyer: Optional[str]
    message_from_payment: Optional[str]
    is_paid: bool
    is_shipped: bool
    create_timestamp: int
    update_timestamp: int
    is_gift: bool
    gift_message: Optional[str]
    grandtotal: Price
    subtotal: Price
    total_price: Price
    total_shipping_cost: Price
    total_tax_cost: Price
    total_vat_cost: Price
    discount_amt: Price
    gift_wrap_price: Price
    shipments: Optional[List[Shipment]]
    transactions: List[Transaction]

    @property
    def country_name(self):
        return pycountry.countries.get(alpha_2=self.country_iso).name



from pydantic import BaseModel
from etsy_apiv3.models.Price import Price

class ShippingProfileUpgrade(BaseModel):
    shipping_profile_id: int
    upgrade_id: int
    upgrade_name: str
    type: str
    rank: int
    language: str
    price: Price
    secondary_price: Price
    shipping_carrier_id: int
    mail_class: str
    min_delivery_days: int
    max_delivery_days: int


from dataclasses import dataclass
from etsy_apiv3.utils.APIV3 import EtsyAuth
from etsy_apiv3.utils.Response import Response
from etsy_apiv3.models.Transaction import Transaction


@dataclass
class TransactionResource():
    auth: EtsyAuth
    __endpoint = "shops/{shop_id}"
    
    def get_transactions_by_listing_id(self, shop_id: int, listing_id: int, limit=25, offset=0) -> Response[Transaction]:
        url = f"{self.__endpoint.format(shop_id=shop_id)}/listings/{listing_id}/transactions"

        json = self.auth.request(url, params={"limit":limit, "offset":offset})
        return Response[Transaction](**json)
    
    def get_transactions_by_receipt_id(self, shop_id: int, receipt_id: int) -> Response[Transaction]:
        url = self.__endpoint.format(shop_id=shop_id) + f"/receipts/{receipt_id}/transactions"
        json = self.auth.request(url)
        
        return Response[Transaction](**json)

    def find_one(self, shop_id: int, transaction_id: int) -> Transaction:
        url = self.__endpoint.format(shop_id=shop_id) + f"/transactions/{transaction_id}"
        json = self.auth.request(url)
        return Transaction(**json)
    
    def find(self, shop_id: int):
        url = self.__endpoint.format(shop_id=shop_id) + f"/transactions"
        json = self.auth.request(url)
        return Response[Transaction](**json)


from __future__ import annotations
from typing import Optional
from pydantic import BaseModel


class ListingImage(BaseModel):
    listing_image_id: int
    hex_code: Optional[str]
    red: Optional[int]
    green: Optional[int]
    blue: Optional[int]
    hue: Optional[int]
    saturation: Optional[int]
    brightness: Optional[int]
    is_black_and_white: Optional[bool]
    creation_tsz: int
    rank: int
    url_75x75: str
    url_170x135: str
    url_570xN: str
    url_fullxfull: str
    full_height: Optional[str]
    full_width: Optional[str]
    alt_text: Optional[str]


import setuptools
from setuptools import find_packages
 
setuptools.setup(
    name="etsy-apiv3",  
    version="1.2.2",
    author="Esat Yılmaz",
    author_email="esatyilmaz3500@gmail.com",
    description="Etsy APIV3 SDK",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5.5',
    install_requires=[
        "pydantic", "requests", "requests_oauthlib", "pycountry"
    ]
)



from dataclasses import dataclass
from etsy_apiv3.models.ListingProperty import ListingProperty
from etsy_apiv3.utils.APIV3 import EtsyAuth
from etsy_apiv3.utils.Response import Response
from etsy_apiv3.models.Listing import Listing

@dataclass
class ListingResource:
    
    auth: EtsyAuth
    
    def find_one(self, listing_id, includes=""):
        
        endpoint = f"listings/{listing_id}"
        json = self.auth.request(endpoint, params={"includes":includes})
        return Listing(**json)

    def find_all_active_listings(self, limit=25, offset=0, keywords="", sort_on="created", sort_order="desc", min_price=None, max_price=None, taxonomy_id=None, shop_location="US"):
        """
        Find All Active Listings By Keywords, Min price, Max Price Or Shop Location

        Args:
            limit (int, optional): Result Limit. Defaults to 25, Max 100.
            offset (int, optional): Result Offset. Defaults to 0.
            keywords (str, optional): Keywords For Active Listings. Defaults to "".
            sort_on (str, optional): Sort By Result. Defaults to "created".
            sort_order (str, optional): Sort By Order. Defaults to "desc".
            min_price (_type_, optional): Min Price. Defaults to None.
            max_price (_type_, optional): Max Price. Defaults to None.
            taxonomy_id (_type_, optional): Taxonomy Id. Defaults to None.
            shop_location (_type_, optional): Shop Location. Defaults to US.

        Returns:
            Response[Listing]: List of Listing Items
        """
        
        endpoint = f"listings/active"
        params = {
            "limit":limit, "offset":offset, "keywords":keywords, "sort_on":sort_on,
            "sort_order":sort_order, "min_price":min_price, "max_price":max_price,
            "taxonomy_id":taxonomy_id, "shop_location":shop_location
        }
        
        json = self.auth.request(endpoint, params=params)

        return Response[Listing](**json)
    
    
    def find_all_active_listings_by_shop(self, shop_id: int, limit: int = 25, sort_on: str = "created", sort_order: str = "desc", offset: int = 0, keywords: str = ""):
        endpoint = f"shops/{shop_id}/listings/active"
        
        params = {"limit": limit, "sort_on": sort_on, "sort_order": sort_order,
                  "offset": offset, "keywords": keywords
        }
        
        response = self.auth.request(endpoint, params=params)
        return Response[Listing](**response)
        
    def get_listings_by_receipt_id(self, shop_id: int, receipt_id: int, limit: int = 25, offset: int = 0) -> Response[Listing]:
        endpoint = f"shops/{shop_id}/receipts/{receipt_id}/listings"
        params = {"limit": limit, "offset": offset}
        
        response = self.auth.request(endpoint, params=params)
        return Response[Listing](**response)
    
    def get_listing_properties(self, shop_id: int, listing_id: int):
        endpoint = f"shops/{shop_id}/listings/{listing_id}/properties"
        
        response = self.auth.request(endpoint)
        return Response[ListingProperty](**response)
    
    def get_listing_property(self, listing_id: int, property_id: int):
        endpoint = f"listings/{listing_id}/properties/{property_id}"
        
        response = self.auth.request(endpoint)
        return ListingProperty(**response)
    
    